//*********************************************************
// This is a modified version of the glfont API by Brad
// Fish
// You can download the original version from:
//
//   Web: http://www.netxs.net/bfish
//   E-mail: bfish@netxs.net
//
// The following modifications are made by Christian Marten
// in order to make this  great toolkit more portable and
// flexible:
//  - GetShort() to get the same
//    results on BigEndian and LittleEndian maschines
//  - glFontCreateASCII() to read an ASCII version of
//    a glFont. The reason for this extention are different
//    binary representations of floats on Intel and MIPS
//    microprocessors.
//  - Use of mipmap in glFontCreateASCII().
//  - glFontConvert() converts the binary version of a glFont
//    to ASCII
//  - glStringSize() returns the width and height of a string
//    in global GL coordinates.
//
//*********************************************************

#ifdef OS_WIN32
#  include <windows.h>
#endif
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "glfont.h"

//*********************************************************
//Variables
//*********************************************************

//Current font
GLFONT *glFont;

//*********************************************************
//Functions
//*********************************************************
/*
char GetByte (FILE *f)
{
  unsigned char b;
  fread (&b, 1, 1, f);
  return b;
}
*/
short GetShort (FILE *f)
{
  unsigned char b[2];
  fread (b, 2,1, f);
  return (b[1]<<8)+(b[0]);
}
/*
long GetLong (FILE *f)
{
  unsigned char b[4];
  fread (b, 4,1,f);
  return (b[3]<<24)+(b[2]<<16)+(b[1]<<8)+(b[0]);
}
*/

//*********************************************************
int glFontCreate (GLFONT *Font, char *FileName, int Tex)
{
	FILE *Input;
	char *TexBytes;
	int   Num;
   short dummy;

	//Open font file
	if ((Input = fopen(FileName, "rb")) == NULL)
		return FALSE;

	//Read glFont structure
   Font->Tex      = GetShort (Input);
   Font->TexWidth = GetShort (Input);
   Font->TexHeight= GetShort (Input);
   Font->IntStart = GetShort (Input);
   Font->IntEnd   = GetShort (Input);
   dummy          = GetShort (Input);

	//Save texture number
	Font->Tex = Tex;

	//Get number of characters
	Num = Font->IntEnd - Font->IntStart + 1;

	//Allocate memory for characters
	if ((Font->Char = (GLFONTCHAR*) malloc(sizeof(GLFONTCHAR)*Num))==NULL)
           return FALSE;

	//Read glFont characters
	fread(Font->Char, sizeof (GLFONTCHAR), Num, Input);

	//Get texture size
	Num = Font->TexWidth * Font->TexHeight * 2;

	//Allocate memory for texture data
	if ((TexBytes = (char *)malloc(Num)) == NULL)
		return FALSE;

	//Read texture data
	fread(TexBytes, sizeof(char), Num, Input);

	//Set texture attributes
	glBindTexture(GL_TEXTURE_2D, Font->Tex);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,
		GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,
		GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
		GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
		GL_LINEAR);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	//Create texture
	glTexImage2D(GL_TEXTURE_2D, 0, 2, Font->TexWidth,
		Font->TexHeight, 0, GL_LUMINANCE_ALPHA,
		GL_UNSIGNED_BYTE, (void *)TexBytes);

	//Clean up
	free(TexBytes);
	fclose(Input);

	//Return pointer to new font
	return TRUE;
}

//*********************************************************
// glFontCreateASCI does the same job as glFontCreate, but
// uses ASCII fonts. This should work with MIPS, SPARC
// and other microprocessors as well.
int glFontCreateASCII (GLFONT *Font, char *FileName, int Tex, int useMipmap,
                      int minFilter, int magFilter)
{
	FILE *f;
	char *TexBytes;
	int   Num, i;

	//Open font file
	if ((f = fopen(FileName, "rb")) == NULL)
	   return 0;

	//Read glFont structure
   fscanf (f, "%d", &Font->Tex);
   fscanf (f, "%d", &Font->TexWidth);
   fscanf (f, "%d", &Font->TexHeight);
   fscanf (f, "%d", &Font->IntStart);
   fscanf (f, "%d", &Font->IntEnd);

	//Save texture number
	Font->Tex = Tex;

	//Get number of characters
	Num = Font->IntEnd - Font->IntStart + 1;

	//Allocate memory for characters
	if ((Font->Char = (GLFONTCHAR*) malloc(sizeof(GLFONTCHAR)*Num))==NULL)
      return FALSE;

	//Read glFont characters
 	for (i=0; i<Num; i++)
           fscanf (f, "%f,%f,%f,%f,%f,%f\n",
                   &Font->Char[i].dx, &Font->Char[i].dy,
                   &Font->Char[i].tx1, &Font->Char[i].ty1,
                   &Font->Char[i].tx2, &Font->Char[i].ty2);

	//Get texture size
	Num = Font->TexWidth * Font->TexHeight * 2;

	//Allocate memory for texture data
	if ((TexBytes = (char *)malloc(Num)) == NULL)
		return FALSE;

	//Read texture data
	fread(TexBytes, sizeof(char), Num, f);

	//Set texture attributes
	glBindTexture(GL_TEXTURE_2D, Font->Tex);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,minFilter);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,magFilter);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	//Create texture
	if (useMipmap == 0)
	   glTexImage2D(GL_TEXTURE_2D, 0, 2, Font->TexWidth,Font->TexHeight,
			0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE,
			(void *)TexBytes);
	else
	   gluBuild2DMipmaps(GL_TEXTURE_2D,GL_LUMINANCE_ALPHA,Font->TexWidth,
			Font->TexHeight, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE,
			(void *)TexBytes);

	//Clean up
	free(TexBytes);
	fclose(f);

	//Return pointer to new font
	return TRUE;
}

//*********************************************************
// glFontConvert extention, cmar, 10/12/2001
// This function takes a glFont file generated by
// glFont.exe and writes a new file with an ASCII
// representation of the font.
// Since the binary format of glFont Files is i386
// specific, this function will only work on i386 maschines
#ifdef I386
int glFontConvert (char *oldFile, char *newFile)
{
   FILE *hIn, *hOut;
   GLFONT font;
   GLFONT *Font=&font;
	char *TexBytes;
	int Num,i;

	//Open font file
	if ((hIn = fopen(oldFile, "rb")) == NULL)
		return FALSE;

	if ((hOut = fopen(newFile, "wt")) == NULL)
		return FALSE;

	//Read glFont structure
	fread(Font, sizeof(GLFONT), 1, hIn);

	fprintf (hOut, "%d\n%d\n%d\n%d\n%d\n", Font->Tex,
		Font->TexWidth, Font->TexHeight,
		Font->IntStart, Font->IntEnd);

	//Get number of characters
	Num = Font->IntEnd - Font->IntStart + 1;

	//Allocate memory for characters
	if ((Font->Char = (GLFONTCHAR *)malloc(sizeof(GLFONTCHAR) * Num)) == NULL)
		return FALSE;

	//Read glFont characters
	fread(Font->Char, sizeof(GLFONTCHAR), Num, hIn);

	for (i=0; i<Num; i++) {
	   fprintf (hOut, "%f,%f,%f,%f,%f,%f\n",
		Font->Char[i].dx, Font->Char[i].dy,
		Font->Char[i].tx1, Font->Char[i].ty1,
               Font->Char[i].tx2, Font->Char[i].ty2);
	}

	//Get texture size
	Num = Font->TexWidth * Font->TexHeight * 2;

	//Allocate memory for texture data
	if ((TexBytes = (char *)malloc(Num)) == NULL)
		return FALSE;

	//Read texture data
	fread(TexBytes, sizeof(char), Num, hIn);

	fwrite(TexBytes, sizeof(char), Num, hOut);

	//Clean up
	free(TexBytes);
	fclose(hIn);
	fclose(hOut);

	return TRUE;
}
#endif
//*********************************************************
void glFontDestroy (GLFONT *Font)
{
	//Free character memory
	free(Font->Char);
}
//*********************************************************
void glFontBegin (GLFONT *Font)
{
	//Save pointer to font structure
	if (Font->Char != NULL)
		glFont = Font;
	else
		glFont = NULL;

	//Bind to font texture
	glBindTexture(GL_TEXTURE_2D, Font->Tex);
}
//*********************************************************
void glFontEnd (void)
{
	//Font no longer current
	glFont = NULL;
}
//*********************************************************
void glFontTextOut (char *String, float x, float y,
	float z)
{
	int Length, i;
	GLFONTCHAR *Char;

	//Return if we don't have a valid glFont
	if (glFont == NULL || String==NULL)
		return;

	//Get length of string
	Length = strlen(String);

	//Begin rendering quads
	glBegin(GL_QUADS);

	//Loop through characters
	for (i = 0; i < Length; i++)
	{
		//Get pointer to glFont character
		Char = &glFont->Char[(int)String[i] -
			glFont->IntStart];

		//Specify vertices and texture coordinates
		glTexCoord2f(Char->tx1, Char->ty1);
		glVertex3f(x, y, z);
		glTexCoord2f(Char->tx1, Char->ty2);
		glVertex3f(x, y - Char->dy, z);
		glTexCoord2f(Char->tx2, Char->ty2);
		glVertex3f(x + Char->dx, y - Char->dy, z);
		glTexCoord2f(Char->tx2, Char->ty1);
		glVertex3f(x + Char->dx, y, z);

		//Move to next character
		x += Char->dx;
	}

	//Stop rendering quads
	glEnd();
}

//*********************************************************
// glStringSize extention, cmar, 10/12/2001
// Function returns width and height of a given string,
// measured in global coordinates
// Koordinaten zurueck.
void glStringSize (char *String, float *dx, float *dy)
{
   int Length, i;
   GLFONTCHAR *Char;

   //Return if we don't have a valid glFont
   if (glFont == NULL)
      return;

   Length = strlen(String);

   *dx=*dy=0;

   if (Length==0)
     return;

   //Loop through characters
   for (i = 0; i < Length; i++)	{
      //Get pointer to glFont character
      Char = &glFont->Char[(int)String[i] - glFont->IntStart];

      *dx += Char->dx;
      if (Char->dy>*dy)
         *dy = Char->dy;
   }
}

//End of file

